classdef LxS < handle
   %% Properties
   properties(Constant)
       DLL_PATH     = fullfile(cd,'\LxS_Lib\LxS_Lib.dll');
   end
   
   properties(Access = public)
        lxs             % Instanz von LxS_Device
        lpsXData
        lpsZData
        lpsScanNumber
   end   
   properties(Access = private)
       lpsScanNumberX
       lpsScanNumberZ
   end
   
   events
      DataSetReceived 
   end
   
   %% Public Methods
   methods
       %% Constructor
        function obj = LxS(ipaddress, LxSPortNo, PCPortNo)
            %IP_ADDRESS    = '192.168.60.3';
            %LPS_PORT     = 9008;
            %PC_PORT      = 5634;            
            
            import System.Net.*; % IPEndPoint(IPAddress.Parse('...'), ...);
            import LxS_Library.*
            NET.addAssembly(obj.DLL_PATH);
                        
            ipendpoint = IPEndPoint(IPAddress.Parse(ipaddress), LxSPortNo);
            obj.lxs =  LxS_Library.LxS_Device(PCPortNo, ipendpoint);            
            obj.lxs.TimeOut = 500;
            addlistener(obj.lxs, 'DataHandler', @obj.dataReceived_CB);
        end
        
        % Destructor
        function deleteLxS(obj)
            if ~isempty(obj.lxs)
                try %#ok
                    obj.lxs.Dispose();
                end
            end
            delete(obj)
        end
        
        %% Connection
        function isConnected = connect(obj)
            isConnected = false;
            if ~isempty(obj.lxs)
                isConnected = obj.lxs.Connect;
                if ~isConnected
                    obj.disconnect;
                    isConnected = obj.lxs.Connect;
                end
            end
            disp(['Connect: ' num2str(isConnected)]);
        end
        
        function disconnect(obj)
            if ~isempty(obj.lxs)
                try %#ok
                    obj.lxs.Disconnect();
                end
            end
        end
        
        %% Sensor Modes
        function sensorMode = getSensorMode(obj)
            sensorMode = LxS_Library.LxS_Mode.INVALID;
            if ~isempty(obj.lxs)
                sensorMode = obj.lxs.GetSensorMode();
            end
        end
        
        function ret = setCommandMode(obj)
            import LxS_Library.*
            ret = false;
            if ~isempty(obj.lxs)
                ret = obj.lxs.SetSensorMode(LxS_Mode.COMMAND);
            end
            disp(['Set Command mode: ' num2str(ret)]);
        end
        
        function ret = setMeasurementMode(obj)
            import LxS_Library.*
            ret = false;
            if ~isempty(obj.lxs)
                ret = obj.lxs.SetSensorMode(LxS_Mode.MEASURE);
            end
            disp(['Set Measurement mode: ' num2str(ret)]);
        end
                
        %% Trigger
        function trigger(obj)
            if ~isempty(obj.lxs)
                obj.lxs.EthernetTrigger();
            end
        end
                
        %% Inspection Tasks
        function ret = setInspectionTask(obj, inspTask)
            ret = false;
            if ~isempty(obj.lxs)
                ret = obj.lxs.SetInspectionTask(uint8(inspTask),true);
            end
        end
        
        function actInspTask = getInspectionTask(obj)
            actInspTask = 0;
            if ~isempty(obj.lxs)
                actInspTask = obj.lxs.GetInspectionTask();
            end
        end
        
        function taskName = getInspectionTaskName(obj, inspTaskNo)
            taskName = ' ';
            if ~isempty(obj.lxs)
                config = obj.lxs.GetDeviceConfiguration;
                %taskName = char(config.Tasks(int16(inspTaskNo)+1).GetParameterByName(LxS_Library.LxS_ParamName.INSP_TASK_NAME).GetField.uint16); %ML 2011
                taskName = char(config.Tasks.Get(int16(inspTaskNo)).GetParameterByName(LxS_Library.LxS_ParamName.INSP_TASK_NAME).GetField.uint16);
            end
        end
        
        %% set and get Single Inspection Task Parameter 

        function setSingleTaskParameter(obj, paramID, paramValue, permanentFlag)
            if ~isempty(obj.lxs)
                paramValue_NET = NET.createArray('System.UInt16',1);
                paramValue_NET.Set(0, uint16(paramValue(1)));
                obj.lxs.SetSingleTaskParameter(uint16(paramID), paramValue_NET, permanentFlag);
            end
        end
        
        function expTime = getSingleTaskParameter(obj, paramID)
            expTime = 1;
            if ~isempty(obj.lxs)
                expTime_Param = obj.lxs.GetSingleTaskParameter(uint16(paramID));
                expTimeArray = expTime_Param.GetField.uint16;
                expTime = expTimeArray(1);
            end
        end
        
        %% Sensor Configuration
        function readConfigFromXMLSendToSensor(obj, file)
            if ~isempty(obj.lxs)
                selInspTask = obj.getInspectionTask;
                
                config = obj.lxs.GetDeviceConfiguration;
                config.ReadConfigurationFromXml(file);
                obj.lxs.SetDeviceConfiguration(config);
                
                obj.setInspectionTask(selInspTask);
            end
        end
        
        function readConfigFromSensorWriteToFile(obj, file)
            if ~isempty(obj.lxs)
                config = obj.lxs.GetDeviceConfiguration;
                config.WriteConfigurationToXml(file);
            end
        end
   end
 
 %% Private Methods
   methods(Access = private)
       %% Data Handler
       function dataReceived_CB(obj, source, arg) %#ok
           if arg.Command == LxS_Library.LxS_Cmd.Z_COORDINATES% strcmp(char(arg.Command), 'Z_COORDINATES')
               obj.lpsZData         = arg.Data.uint16;
               obj.lpsScanNumberZ   = arg.ScanNumber;
           end
           if arg.Command == LxS_Library.LxS_Cmd.X_COORDINATES% strcmp(char(arg.Command), 'X_COORDINATES')
               if arg.ScanNumber == obj.lpsScanNumberZ
                   obj.lpsXData         = typecast(arg.Data.uint16, 'int16');
                   %obj.lpsScanNumberX   = arg.ScanNumber;
                   obj.lpsScanNumber = arg.ScanNumber;
                   notify(obj, 'DataSetReceived')
               end
           end          
       end
   end
end